#!/usr/bin/env Rscript
# The above "shebang" allows this file to be self-executing
#///////////////////////////////////////////////////////////////////////////////
#//                                                                           //
#// This software and its documentation are copyright (c) 2014-2015 by Joshua //
#// N. Burton and the University of Washington.  All rights are reserved.     //
#//                                                                           //
#// THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  //
#// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                //
#// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  //
#// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY      //
#// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT //
#// OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR  //
#// THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                //
#//                                                                           //
#///////////////////////////////////////////////////////////////////////////////



# QuickDotplot.POA.R
#
# The "POA" stands for "PlotOrderAccuracy" and refers to the evaluation function Reporter::PlotOrderAccuracy().
#
# This R script was generated by the Perl command QuickDotplot, at
# Wed Apr 17 10:45:25 2013
# Specifically, the command was:
# < directory=/net/gs/vol2/home/jnburton/vol10/src/HiC >
# QuickDotplot reporter


library( ggplot2, quietly=TRUE ) # ggplot

dotplot.file <- 'QuickDotplot.POA.txt'
jpeg.file <- 'out/POA.jpg'




# Read the dotplot file.
dotplot <- read.table( dotplot.file, header=FALSE )


# Re-order the error types so that the color mapping is consistent between dotplots.
# The error names are defined in Reporter::PlotOrderAccuracy().
err_colors <- c( "awesome", "err_orient", "err_order", "err_order_orient", "err_interchrom", "err_interchrom_orient", "err_interchrom_order", "err_interchrom_order_orient" )
dotplot$colors <- factor( dotplot[,3], levels = err_colors )

dot.size <- ceiling( 50 / sqrt( dim(dotplot)[1] ) )



# Load the data into a ggplot object.
p <- ggplot( dotplot, aes( x=V1, y=V2 ) )
p <- p + theme_bw()

# Plot the points, and load in the colors, if any were given.
p <- p + geom_point( size = dot.size, aes( col=colors ) )
if ( dim( table( dotplot[,3] ) ) <= 10 ) {
    p <- p + scale_colour_manual(values=c('gray75','gold2','#5555FF','#338800','red2','darkorange4','purple','black')) # color map; matches the factor above
}

# Assign text labels.
p <- p + ggtitle( "QuickDotplot showing errors in contig ordering" )
p <- p + xlab("")
p <- p + ylab("")

# Save the plot to the jpeg file.  Adjust the width to make room for a legend, if there is one.
suppressWarnings( ggsave( filename=jpeg.file, plot=p, width=20, height=20 ) )
